//Programme de test de l'intgrateur RK4 symplectique  pas fixe.

import mn.*;
import eqmvt.*;
import java.lang.Math;

/**
 * Programme de test de l'intgrateur RK4 symplectique  pas fixe :
 * tude d'un problme simple (oscillateur harmonique  2D).
 *
 * <ul>
 * <li><b>deltaT</b>        <dd> Pas d'integration </dd>
 * <li><b>precision</b>	    <dd>Prcision demande  la solution</dd>
 * <li><b>relax</b>	        <dd>Paramtre de relaxation de la mthode d'intgration RK4 symplectique  pas fixe</dd>
 * <li><b>param</b>	        <dd>Tableau de paramtres du problme (masse m, raideurs k1 et k2)</dd>
 * <li><b>omega</b>	        <dd>Tableau des sqrt(k_i/m)</dd>
 * <li><b>xx</b>            <dd>Coordonnes (Q1,Q2,P1,P2) calcules avec l'intgrateur</dd>
 * <li><b>x_ana</b>	        <dd>Coordonnes (Q1,Q2,P1,P2) calcules analytiquement</dd>
 * <li><b>x_diff</b>        <dd>Diffrence relative entre les deux solutions</dd>
 * </ul>
 *
 * @author Pierre INIZAN, Mathieu HAVEL 
 * 
 * @version 12/01/2007
 * @since Projet Trois Corps 2007
 * 
 * @see mn.Integrateur
 * @see mn.DetermineZero
 * @see eqmvt.EqMvmtTestIntegrateur
 *
 */

public class TestIntegrateur {

	/**
	 * Intgre les quations du mouvement et compare aux solutions analytiques
	 */
	public static void main(String[] args) {
		
		Integrateur integrateur;    				// pour le calcul de la trajectoire
		InterfaceEqMvmt mouvement;					// pour le calcul de l'integrale de Jacobi
		String fEqMvmt = "EqMvmtTestIntegrateur";  	// nom lie au nom du fichier contenant l'equation du mouvement

		double deltaT;		// pas d'intgration
		double precision;	// prcision demande de la solution
		double relax;		// paramtre de relaxation de la mthode RK4 symplectique

		double[] param = new double[3];		// tableau de paramtres de l'intgrateur (ie. la masse)
		double[] omega = new double[2];		// tableau contenant sqrt(k1/m) et sqrt(k2/m)

		double[] xx = new double[4]; 		// tableau des coordonnes (positions + moments conjugus) calcules
		double[] x_ana = new double[4]; 	// tableau des coordonnes (positions + moments conjugus) analytiques
		double[] x_initial = new double[4]; // tableau des x initiaux
		double[] x_diff = new double[4];	// tableau des diffrences entre calculs intgrateur et calculs analytiques
		double jacobiInitial; 				// valeur de l'integrale de jacobi pour les conditions initiales

		double iterations;	// nombre d'itrations  effectuer
		int appel;			// compteur d'itrations

		// **** INITIALISATION DU PROGRAMME **** //
		// ************************************* //

		// paramtres du problme
		param[0] = 0.5; // masse
		param[1] = 0.4; // constante de raideur 1
		param[2] = 0.3; // constante de raideur 2
		omega[0] = Math.sqrt(param[1]/param[0]);
		omega[1] = Math.sqrt(param[2]/param[0]);

		// conditions initiales du problme
		xx[0] = 0.5; // q1 = x1
		x_initial[0] = xx[0];
		xx[1] = 0.5; // q2 = x2
		x_initial[1] = xx[1];
		xx[2] = 0.02; // p1 = m . x_dot1
		x_initial[2] = xx[2];
		xx[3] = 0.01; // p2 = m . x_dot2
		x_initial[3] = xx[3];

		// paramtres de l'intgrateur
		iterations = 10000;
		deltaT = 0.001;
		precision = 1e-10;
		relax = 1;

		appel = 0;
		
		// **** INTEGRATION des EQUATIONS et COMPARAISON avec SOLUTIONS ANALYTIQUES**** //
		// **************************************************************************** //

		integrateur = new Integrateur(4,param,fEqMvmt);
		mouvement = UsineEqMvmt.nouveau(fEqMvmt);
		jacobiInitial = mouvement.intPrem(xx,param);

		double jacobi;			// intgrale de jacobi absolue
		double jacobiRelatif;	// valeur relative de l'intgrale de Jacobi

		for(int k=0;k<iterations;k++){

			appel++;
			
			// Intgrateur Symplectique
			xx = integrateur.rk4sympf(xx,deltaT,precision,relax);
			jacobi = mouvement.intPrem(xx,param);
			jacobiRelatif = (jacobi - jacobiInitial) / jacobiInitial;
			
			// solutions analytiques
			x_ana[0] = x_initial[0]*Math.cos(omega[0] * deltaT* appel) + x_initial[2]/(omega[0]*param[0])*Math.sin(omega[0] * deltaT* appel);
			x_ana[1] = x_initial[1]*Math.cos(omega[1] * deltaT* appel) + x_initial[3]/(omega[1]*param[0])*Math.sin(omega[1] * deltaT* appel);
			x_ana[2] = param[0]*(-x_initial[0]*omega[0]*Math.sin(omega[0] * deltaT* appel)) + x_initial[2]*Math.cos(omega[0] * deltaT* appel);
			x_ana[3] = param[0]*(-x_initial[1]*omega[1]*Math.sin(omega[1] * deltaT* appel)) + x_initial[3]*Math.cos(omega[1] * deltaT* appel);
			
			// diffrences des rsultats
			x_diff[0] = (xx[0] - x_ana[0])/x_ana[0];
			x_diff[1] = (xx[1] - x_ana[1])/x_ana[1];
			x_diff[2] = (xx[2] - x_ana[2])/x_ana[2];
			x_diff[3] = (xx[3] - x_ana[3])/x_ana[3];
			
			// impression des resultats
			System.out.println(appel+"\t"+x_diff[0]+"\t"+x_diff[1]+"\t"+x_diff[2]+"\t"+x_diff[3]+"\t"+jacobiRelatif);
			//System.out.println(appel+"\t"+xx[0]+"\t"+xx[1]+"\t"+xx[2]+"\t"+xx[3]);//+"\t"+jacobiRelatif);
			//System.out.println(appel+"\t"+x_ana[0]+"\t"+x_ana[1]+"\t"+x_ana[2]+"\t"+x_ana[3]);//+"\t"+jacobiRelatif);
			
		}
	}

}
