package fonction;
import eqmvt.*;

/**
 * Classe qui implmente la matrice jacobienne Dg d'une fonction g hrite de ChoixDeFonction  partir d'une fonction f hrite de InterfaceEqMvmt, telle que Dg(x) = cste_multe*Df(x) - Id, o Df est la matrice jacobienne de f.
 * f et cste_mult sont dfinies dans le constructeur. Dans f, on trouve la fonction elle-mme, ainsi que sa matrice jacobienne associee.
 * <p>
 * Utilisation dans l'intgrateur symplectique (rk4sympf, dans Integrateur.java), o f reprsente l'quation du mouvement, donne dans le constructeur de la classe Integrateur,et g la fonction  annuler par l'algorithme de Newton. Celui-ci prend en entre la fonction g et sa matrice jacobienne Dg. La classe FctNewtSymp implmente la fonction g. 
 * 
 *
 * @see FctNewtSymp
 * @see mn.Integrateur Integrateur
 *
 * @author Mathieu HAVEL, Pierre INIZAN
 * @version 07/01/2007
 * @since Projet Trois Corps 2007
 */




public class JacNewtSymp implements ChoixDeFonction {
	 
    InterfaceEqMvmt f;
    double cste_mult;
    
    //Constructeur  
    /**
     * Constructeur
     * @param f Equation du mouvement
     * @param cste_mult Facteur multiplicatif intervenant dans la dfinition des ksi de l'intgrateur RK4 symp
     */
    public JacNewtSymp(InterfaceEqMvmt f, double cste_mult){
  this.f = f;
  this.cste_mult = cste_mult;
    }
    
    //Methode vide mais ncessaire pour que JacNewtSymp hrite de ChoixDeFonction
    public double fonction(double variable, double[] parametres) {
  //definition de la methode fonction non utilise dans ce cas
  double resultat = 0;
  return resultat;
    }
    

    //Methode vide mais ncessaire pour que JacNewtSymp hrite de ChoixDeFonction
    public double fonction(double[] variable, double[] parametres) {
  //definition de la methode fonction non utilise dans ce cas
  double resultat = 0;
  return resultat;
    }
    

  /**
   * @param variable Position en laquelle on value la fonction
   * @param parametres Paramtres utiliss par la fonction f
   * @param taille Dimension de l'espace d'arrive de la fonction
   * @return Valeur de la fonction value au point "variable"
   */

    //Dfinition de la nouvelle matrice jacobienne Dg, dfinie  partir de Df   
    public double[] fonction_multi(double[] variable, double[] parametres, int taille) {
  int n = variable.length;
  int i,j;
  int m = n * n;
  double[] resultat = new double[m];
  double[] Df;
  Df = f.jacMvmt(variable, parametres); // Jacobienne de f, situee dans la methode jacMvmt, evaluee au point "variable" 
  for(i=0;i<m;i++){
      resultat[i] = cste_mult*Df[i]; //Evaluation de cste_mult*Df(variable)
  }

  for(j=0;j<n;j++){
      resultat[(n+1)*j] -= 1; //On soustrait Id au resultat precedent, pour obtenir cste_mult*Df(variable) - Id
  }

  return resultat; //Renvoie cste_mult*Df(variable) - Id = Dg(variable)
  
    }
   
}
