package vitessenulle;
import mn.*;

/**
 * Classe de calcul des courbes de vitesse nulle du problme a 3 corps restreint  par la mthode de continuation de courbe.
 *
 *  @author Guillaume BOUBIN et Gatan LE CHAT
 *  @version 09/01/2007
 *  @see DetermineZero
 *  @since Projet Trois Corps 2007
 */
public class CourbeVitesse0{

 private static double[] param = new double[2];
 double pasIntegration;

  /** 
   * Constructeur de la classe
   *
   * @param J le Jacobien (param[0])
   * @param mu la masse reduite du systeme (param[1])
   * @param pasInte le pas d'integration maximum utilise par l'integrateur RK4_pv
   *
   */
  public CourbeVitesse0(double J, double mu, double pasInte){
   param[0]=J ;
   param[1]=mu ;
   pasIntegration = pasInte;
  }
    
  /**
   * Mthode de dtermination d'un point d'une courbe vitesse nulle utilisant une mthode de Newton-Raphson (N-R) coupl  une dichotomie pour dterminer le point de dpard de N-R
   *
   * @param origin Position du point autour du quel on veut dterminer un point de la courbe de vitesse nulle
   * @return position (tableau de double contenant q1 et q2) du point de la courbe de vitesse nulle
   *
   */
  public double[] pointV0(double[] origin) {
     
    //**********appel des methodes et initialisation des valeurs utilises
    FoncVitesseNulle fonction = new FoncVitesseNulle();
    DeriveVitesseNulle derive = new DeriveVitesseNulle();
    DetermineZero methode = new DetermineZero();
  
    double[] params = new double[3];
    params[0] = param[0];//<=> J = J
    params[1] = param[1];//<=> mu = mu
    params[2] = origin[0];//<=> params[2] = q1 de l'origine
  
    double precNewton;//precision du zero obtenue par N-R
    precNewton = 1.e-13;
    double precDico;//precision du zero obtenue par dichotomie
    precDico = 1.e-6;
  
    //*************dtermination par dichotomie du point de dpart de Newton-Raphson
    //on utilise la methode par dichotomie pour restreindre  l'intervalle contenant le zero
    //si on par directement de L4 avec la methode de Newton , elle diverge, la dervive etant proche de zero.
    double[] intervalle = new double[2];
    intervalle[1]=origin[1];
    if(origin[1]==0.0) {

        intervalle[0]=-0.5;
        }
     else {
        intervalle[0]=-2.0;
        }
    intervalle=methode.dichotomie(intervalle,precDico,params,fonction);
  
//  -Debugging
//   System.out.println("intervalle : "+intervalle[0]+" , "+intervalle[1]);
//  -Debugging
    
    double q2Init;
    q2Init=intervalle[0];

    q2Init=methode.newtonRaphson(q2Init,precNewton,params,fonction,derive);
    
    double[] qInit = new double[2];
    qInit[0] = origin[0];
    qInit[1] = q2Init;

//  -Debugging    
//   System.out.println("q2Init : "+q2Init);
//  -Debugging
    
    return qInit;
   
   }
   
   /**
   *mthode de continuation pour la courbe de vitesse nulle utilisant un intgrateur de type RK4  pas variable
   *
   * @param position (q1, q2) de la position prcdente
   * @return position (q1, q2) de la position suivante sur la courbe
   */

  public double[] courbeV0(double[] position) {          
  
    //***********Initialisation
    Integrateur monIntegrateur;
    String bidon="bidon";//voir constructeur de Integrateur, dans notre cas pas d'equation du mouvement
    monIntegrateur = new Integrateur(2, param, bidon);
    double[] deplacer = new double[2];
    double precision = 1.e-06;
  
    //*************calcul de la position suivante sur la courbe de vitesse nulle 
    Dqds dQisurdS = new Dqds();
    deplacer=monIntegrateur.rk4pv(position, pasIntegration,precision,dQisurdS, false);
  
    return deplacer;  
  }
  
   /**
   *mthode de dtermination du moment ou l'on peut arreter la continuation de la courbe lorsque celle-ci est fini, car les courbe sont ici des courbes fermes. La methode repose sur le fait que la courbe est fini, lorsqu'un angle de 2 pi a t effectuer autour du centre de la courbe (identique au parametre "origin" utilis dans la mthode "pointV0(double[] origin)".
   *
   *@param xy0 rsultat precedement calcul, initialis a 0
   *@param lagrange point de lagrange (LQ1,LQ2) a l'interieur de la courbe de vitesse nulle
   *@param position point (Q1,Q2) de la courbe calcul precedement par la mthode de continuation 
   *@return position (x,y) de la variable "position" par rapport  "lagrange", ainsi qu'un parametre gale a 1.0 lorsque la courbe est fini, 0.0 autrement.
   */

  public double[] angle(double[] xy0,double[] lagrange,double[] position) {
      
    //*************Calcul de x et y    
    double x;
    double y;
      
    x = position[0] - lagrange[0];
    y = position[1] - lagrange[1];

    double[] stop = new double [3];

    //*************Determination de l'arret ou non
    if ((x*xy0[0]<0)&&(y<0)) {
       stop[0]=x;
       stop[1]=y;
       stop[2]=1.0;
       }
      
    else {
       stop[0]=x;
       stop[1]=y;
       stop[2]=0.0;
       }
      
    return stop;
    }
}
